@@ -1,7 +1,8 @@
 from __future__ import annotations
 
 import asyncio
-from datetime import datetime, timezone
+import contextlib
+from datetime import datetime, timedelta, timezone
 import json
 import logging
 import re
@@ -14,8 +15,8 @@ import zigpy.appdb_schemas
 import zigpy.backups
 import zigpy.device
 import zigpy.endpoint
+import zigpy.exceptions
 import zigpy.group
-import zigpy.neighbor
 import zigpy.profiles
 import zigpy.quirks
 import zigpy.state
@@ -27,13 +28,15 @@ from zigpy.zdo import types as zdo_t
 
 LOGGER = logging.getLogger(__name__)
 
-DB_VERSION = 10
+DB_VERSION = 11
 DB_V = f"_v{DB_VERSION}"
 MIN_SQLITE_VERSION = (3, 24, 0)
 
 UNIX_EPOCH = datetime.fromtimestamp(0, tz=timezone.utc)
 DB_V_REGEX = re.compile(r"(?:_v\d+)?$")
 
+MIN_LAST_SEEN_DELTA = timedelta(seconds=30).total_seconds()
+
 
 def _import_compatible_sqlite3(min_version: tuple[int, int, int]) -> types.ModuleType:
     """
@@ -126,7 +129,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
         # Truncate the SQLite journal file instead of deleting it after transactions
         await self._set_isolation_level(None)
-        await self.execute("PRAGMA journal_mode = TRUNCATE")
+        await self.execute("PRAGMA journal_mode = WAL")
+        await self.execute("PRAGMA synchronous = normal")
+        await self.execute("PRAGMA temp_store = memory")
         await self._set_isolation_level("DEFERRED")
 
         await self.execute("PRAGMA foreign_keys = ON")
@@ -183,7 +188,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
 
         # Delete the journal on shutdown
         await self._set_isolation_level(None)
-        await self.execute("PRAGMA journal_mode = DELETE")
+        await self.execute("PRAGMA wal_checkpoint;")
         await self._set_isolation_level("DEFERRED")
 
         await self._db.close()
@@ -233,9 +238,16 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         self.enqueue("_save_device_last_seen", device.ieee, last_seen)
 
     async def _save_device_last_seen(self, ieee: t.EUI64, last_seen: datetime) -> None:
+        q = f"""UPDATE devices{DB_V}
+                    SET last_seen=:ts
+                    WHERE ieee=:ieee AND :ts - last_seen > :min_last_seen_delta"""
         await self.execute(
-            f"UPDATE devices{DB_V} SET last_seen=? WHERE ieee=?",
-            (last_seen.timestamp(), ieee),
+            q,
+            {
+                "ts": last_seen.timestamp(),
+                "ieee": ieee,
+                "min_last_seen_delta": MIN_LAST_SEEN_DELTA,
+            },
         )
         await self._db.commit()
 
@@ -249,18 +261,16 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         if relays is None:
             await self.execute(f"DELETE FROM relays{DB_V} WHERE ieee = ?", (ieee,))
         else:
-            q = f"""INSERT INTO relays{DB_V} VALUES (?, ?)
+            q = f"""INSERT INTO relays{DB_V} VALUES (:ieee, :relays)
                         ON CONFLICT (ieee)
-                        DO UPDATE SET relays=excluded.relays"""
-            await self.execute(q, (ieee, relays.serialize()))
+                        DO UPDATE SET relays=excluded.relays WHERE relays != :relays"""
+            await self.execute(q, {"ieee": ieee, "relays": relays.serialize()})
 
         await self._db.commit()
 
     def attribute_updated(
         self, cluster: zigpy.typing.ClusterType, attrid: int, value: Any
     ) -> None:
-        if not cluster.endpoint.device.is_initialized:
-            return
 
         self.enqueue(
             "_save_attribute",
@@ -274,8 +284,6 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     def unsupported_attribute_added(
         self, cluster: zigpy.typing.ClusterType, attrid: int
     ) -> None:
-        if not cluster.endpoint.device.is_initialized:
-            return
 
         self.enqueue(
             "_unsupported_attribute_added",
@@ -294,22 +302,58 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self.execute(q, (ieee, endpoint_id, cluster_id, attrid))
         await self._db.commit()
 
-    def neighbors_updated(self, neighbors: zigpy.neighbor.Neighbors) -> None:
-        """Neighbor update from ZDO_Lqi_rsp."""
-        self.enqueue("_neighbors_updated", neighbors)
+    def unsupported_attribute_removed(
+        self, cluster: zigpy.typing.ClusterType, attrid: int
+    ) -> None:
 
-    async def _neighbors_updated(self, neighbors: zigpy.neighbor.Neighbors) -> None:
-        await self.execute(
-            f"DELETE FROM neighbors{DB_V} WHERE device_ieee = ?", (neighbors.ieee,)
+        self.enqueue(
+            "_unsupported_attribute_removed",
+            cluster.endpoint.device.ieee,
+            cluster.endpoint.endpoint_id,
+            cluster.cluster_id,
+            attrid,
         )
 
-        rows = [(neighbors.ieee,) + n.neighbor.as_tuple() for n in neighbors.neighbors]
+    async def _unsupported_attribute_removed(
+        self, ieee: t.EUI64, endpoint_id: int, cluster_id: int, attrid: int
+    ) -> None:
+        q = f"""DELETE FROM unsupported_attributes{DB_V} WHERE ieee = ?
+                                                         AND endpoint_id = ?
+                                                         AND cluster = ?
+                                                         AND attrid = ?"""
+        await self.execute(q, (ieee, endpoint_id, cluster_id, attrid))
+        await self._db.commit()
+
+    def neighbors_updated(self, ieee: t.EUI64, neighbors: list[zdo_t.Neighbor]) -> None:
+        """Neighbor update from Mgmt_Lqi_req."""
+        self.enqueue("_neighbors_updated", ieee, neighbors)
+
+    async def _neighbors_updated(
+        self, ieee: t.EUI64, neighbors: list[zdo_t.Neighbor]
+    ) -> None:
+        await self.execute(f"DELETE FROM neighbors{DB_V} WHERE device_ieee = ?", [ieee])
+
+        rows = [(ieee,) + neighbor.as_tuple() for neighbor in neighbors]
 
         await self._db.executemany(
             f"INSERT INTO neighbors{DB_V} VALUES (?,?,?,?,?,?,?,?,?,?,?,?)", rows
         )
         await self._db.commit()
 
+    def routes_updated(self, ieee: t.EUI64, routes: list[zdo_t.Route]) -> None:
+        """Route update from Mgmt_Rtg_req."""
+        self.enqueue("_routes_updated", ieee, routes)
+
+    async def _routes_updated(self, ieee: t.EUI64, routes: list[zdo_t.Route]) -> None:
+        await self.execute(f"DELETE FROM routes{DB_V} WHERE device_ieee = ?", [ieee])
+
+        rows = [(ieee,) + route.as_tuple() for route in routes]
+
+        await self._db.executemany(
+            f"INSERT INTO routes{DB_V} VALUES (?,?,?,?,?,?,?,?)", rows
+        )
+        await self._db.commit()
+
     def group_added(self, group: zigpy.group.Group) -> None:
         """Group is added."""
         self.enqueue("_group_added", group)
@@ -494,7 +538,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         q = f"""INSERT INTO attributes_cache{DB_V} VALUES (?, ?, ?, ?, ?)
                     ON CONFLICT (ieee, endpoint_id, cluster, attrid)
                     DO UPDATE SET
-                        value=excluded.value"""
+                        value=excluded.value WHERE value != excluded.value"""
         await self.execute(q, (ieee, endpoint_id, cluster_id, attrid, value))
         await self._db.commit()
 
@@ -539,6 +583,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         await self._load_group_members()
         await self._load_relays()
         await self._load_neighbors()
+        await self._load_routes()
         await self._load_network_backups()
         await self._register_device_listeners()
 
@@ -660,10 +705,14 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _load_neighbors(self) -> None:
         async with self.execute(f"SELECT * FROM neighbors{DB_V}") as cursor:
             async for ieee, *fields in cursor:
-                dev = self._application.get_device(ieee)
                 neighbor = zdo_t.Neighbor(*fields)
-                assert neighbor.is_valid
-                dev.neighbors.add_neighbor(neighbor)
+                self._application.topology.neighbors[ieee].append(neighbor)
+
+    async def _load_routes(self) -> None:
+        async with self.execute(f"SELECT * FROM routes{DB_V}") as cursor:
+            async for ieee, *fields in cursor:
+                route = zdo_t.Route(*fields)
+                self._application.topology.routes[ieee].append(route)
 
     async def _load_network_backups(self) -> None:
         self._application.backups.backups.clear()
@@ -671,16 +720,34 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
         async with self.execute(
             f"SELECT * FROM network_backups{DB_V} ORDER BY id"
         ) as cursor:
+            backups = []
+
             async for id_, backup_json in cursor:
                 backup = zigpy.backups.NetworkBackup.from_dict(json.loads(backup_json))
-                self._application.backups.add_backup(backup)
+                backups.append(backup)
+
+        backups.sort(key=lambda b: b.backup_time)
+
+        for backup in backups:
+            self._application.backups.add_backup(backup, suppress_event=True)
 
     async def _register_device_listeners(self) -> None:
         for dev in self._application.devices.values():
             dev.add_context_listener(self)
-            dev.neighbors.add_context_listener(self)
 
-    async def _get_table_versions(self) -> dict[str, str]:
+    @contextlib.asynccontextmanager
+    async def _transaction(self):
+        await self.execute("BEGIN TRANSACTION")
+
+        try:
+            yield
+        except Exception:
+            await self.execute("ROLLBACK")
+            raise
+        else:
+            await self.execute("COMMIT")
+
+    async def _get_table_versions(self) -> dict[str, int]:
         tables = {}
 
         async with self.execute(
@@ -695,26 +762,44 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 match = DB_V_REGEX.search(name)
                 assert match is not None
 
-                tables[name] = match.group(0)
+                tables[name] = int(match.group(0)[2:] or "0")
 
         return tables
 
     async def _table_exists(self, name: str) -> bool:
         return name in (await self._get_table_versions())
 
-    async def _run_migrations(self):
-        """Migrates the database to the newest schema."""
+    async def _run_migrations(self) -> bool:
+        """
+        Migrates the database to the newest schema, returning True if migrations ran.
+        """
+
+        tables = await self._get_table_versions()
+        tables_version = max(tables.values(), default=0)
 
         async with self.execute("PRAGMA user_version") as cursor:
             (db_version,) = await cursor.fetchone()
 
-        LOGGER.debug("Current database version is v%s", db_version)
+        LOGGER.debug(
+            "Current database version is v%s (table version v%s)",
+            db_version,
+            tables_version,
+        )
 
-        # Very old databases did not set `user_version` but still should be migrated
-        if db_version == 0 and not await self._table_exists("devices"):
+        # Table version suffixes were introduced in v4. If the table version suffix does
+        # not match `user_version`, either zigpy was downgraded to a *really* old
+        # version (July 2021), or it's corrupt. Running migrations could delete existing
+        # table data, and since we cannot guarantee the schema is intact, fail early.
+        if tables_version >= 4 and tables_version != db_version:
+            raise zigpy.exceptions.CorruptDatabase(
+                f"The `zigbee.db` database version ({db_version}) does not match its"
+                f" max table version ({tables_version}). The database is inconsistent.",
+            )
+
+        if db_version == 0 and not tables:
             # If this is a brand new database, just load the current schema
             await self.executescript(zigpy.appdb_schemas.SCHEMAS[DB_VERSION])
-            return
+            return False
         elif db_version > DB_VERSION:
             LOGGER.error(
                 "This zigpy release uses database schema v%s but the database is v%s."
@@ -723,12 +808,10 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 DB_VERSION,
                 db_version,
             )
-            return
+            return False
 
         # All migrations must succeed. If any fail, the database is not touched.
-        await self.execute("BEGIN TRANSACTION")
-
-        try:
+        async with self._transaction():
             for migration, to_db_version in [
                 (self._migrate_to_v4, 4),
                 (self._migrate_to_v5, 5),
@@ -737,6 +820,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 (self._migrate_to_v8, 8),
                 (self._migrate_to_v9, 9),
                 (self._migrate_to_v10, 10),
+                (self._migrate_to_v11, 11),
             ]:
                 if db_version >= min(to_db_version, DB_VERSION):
                     continue
@@ -748,11 +832,8 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 await migration()
 
                 db_version = to_db_version
-        except Exception:
-            await self.execute("ROLLBACK")
-            raise
-        else:
-            await self.execute("COMMIT")
+
+        return True
 
     async def _migrate_tables(
         self, table_map: dict[str, str], *, errors: str = "raise"
@@ -799,7 +880,7 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                             pass
                         else:
                             raise ValueError(
-                                f"Invalid value for `errors`: {errors}!r"
+                                f"Invalid value for `errors`: {errors!r}"
                             )  # noqa
 
     async def _migrate_to_v4(self):
@@ -841,10 +922,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _migrate_to_v5(self):
         """Schema v5 introduced global table version suffixes and removed stale rows"""
 
-        # Copy the devices table first, it should have no conflicts
-        await self.execute("INSERT INTO devices_v5 SELECT * FROM devices")
         await self._migrate_tables(
             {
+                "devices": "devices_v5",
                 "endpoints": "endpoints_v5",
                 "clusters": "in_clusters_v5",
                 "output_clusters": "out_clusters_v5",
@@ -856,7 +936,6 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 "neighbors_v4": "neighbors_v5",
                 "node_descriptors_v4": "node_descriptors_v5",
                 # Explicitly specify which tables will not be migrated
-                "devices": None,
                 "neighbors": None,
                 "node_descriptors": None,
             },
@@ -866,10 +945,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _migrate_to_v6(self):
         """Schema v6 relaxed the `attribute_cache` table schema to ignore endpoints"""
 
-        # Copy the devices table first, it should have no conflicts
-        await self.execute("INSERT INTO devices_v6 SELECT * FROM devices_v5")
         await self._migrate_tables(
             {
+                "devices_v5": "devices_v6",
                 "endpoints_v5": "endpoints_v6",
                 "in_clusters_v5": "in_clusters_v6",
                 "out_clusters_v5": "out_clusters_v6",
@@ -879,7 +957,6 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 "attributes_cache_v5": "attributes_cache_v6",
                 "neighbors_v5": "neighbors_v6",
                 "node_descriptors_v5": "node_descriptors_v6",
-                "devices_v5": None,
             }
         )
 
@@ -916,10 +993,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _migrate_to_v7(self):
         """Schema v7 added the `unsupported_attributes` table."""
 
-        # Copy the devices table first, it should have no conflicts
-        await self.execute("INSERT INTO devices_v7 SELECT * FROM devices_v6")
         await self._migrate_tables(
             {
+                "devices_v6": "devices_v7",
                 "endpoints_v6": "endpoints_v7",
                 "in_clusters_v6": "in_clusters_v7",
                 "out_clusters_v6": "out_clusters_v7",
@@ -929,7 +1005,6 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 "attributes_cache_v6": "attributes_cache_v7",
                 "neighbors_v6": "neighbors_v7",
                 "node_descriptors_v6": "node_descriptors_v7",
-                "devices_v6": None,
             }
         )
 
@@ -988,10 +1063,9 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
     async def _migrate_to_v10(self):
         """Schema v10 added a new `network_backups_v10` table."""
 
-        await self.execute("INSERT INTO devices_v10 SELECT * FROM devices_v9")
-
         await self._migrate_tables(
             {
+                "devices_v9": "devices_v10",
                 "endpoints_v9": "endpoints_v10",
                 "in_clusters_v9": "in_clusters_v10",
                 "out_clusters_v9": "out_clusters_v10",
@@ -1002,6 +1076,25 @@ class PersistingListener(zigpy.util.CatchingTaskMixin):
                 "neighbors_v9": "neighbors_v10",
                 "node_descriptors_v9": "node_descriptors_v10",
                 "unsupported_attributes_v9": "unsupported_attributes_v10",
-                "devices_v9": None,
+            }
+        )
+
+    async def _migrate_to_v11(self):
+        """Schema v11 added a new `routes_v11` table."""
+
+        await self._migrate_tables(
+            {
+                "devices_v10": "devices_v11",
+                "endpoints_v10": "endpoints_v11",
+                "in_clusters_v10": "in_clusters_v11",
+                "out_clusters_v10": "out_clusters_v11",
+                "groups_v10": "groups_v11",
+                "group_members_v10": "group_members_v11",
+                "relays_v10": "relays_v11",
+                "attributes_cache_v10": "attributes_cache_v11",
+                "neighbors_v10": "neighbors_v11",
+                "node_descriptors_v10": "node_descriptors_v11",
+                "unsupported_attributes_v10": "unsupported_attributes_v11",
+                "network_backups_v10": "network_backups_v11",
             }
         )
