@@ -13,6 +13,10 @@ class APIException(ZigbeeException):
 class DeliveryError(ZigbeeException):
     """Message delivery failed in some way"""
 
+    def __init__(self, message: str, status=None):
+        super().__init__(message)
+        self.status = status
+
 
 class InvalidResponse(ZigbeeException):
     """A ZDO or ZCL response has an unsuccessful status code"""
@@ -22,9 +26,21 @@ class RadioException(Exception):
     """Base exception class for radio exceptions"""
 
 
+class TransientConnectionError(RadioException):
+    """Connection to the radio failed but will likely succeed in the near future"""
+
+
 class NetworkNotFormed(RadioException):
     """A network cannot be started because the radio has no stored network info"""
 
 
 class FormationFailure(RadioException):
     """Network settings could not be written to the radio"""
+
+
+class NetworkSettingsInconsistent(ZigbeeException):
+    """Loaded network settings are different from what is in the database"""
+
+
+class CorruptDatabase(ZigbeeException):
+    """The SQLite database is corrupt or otherwise inconsistent"""
